@@ -1,15 +1,16 @@
 ## File Name: logLik_tam.R
-## File Version: 9.02
+## File Version: 9.04
 
 
 ###############################################################
-# log-likelihood function tam.mml 
-logLik.tam <- function (object, ...) {
-	# extract log-likelihood
-	out <- - object$ic$deviance / 2 
+# log-likelihood function tam.mml
+logLik.tam <- function (object, ...)
+{
+    # extract log-likelihood
+    out <- - object$ic$deviance / 2
     # number of parameters
     attr(out, "df") <- object$ic$Npars
-	# extract number of observations
+    # extract number of observations
     attr(out, "nobs") <- object$ic$n
     class(out) <- "logLik"
     return(out)
